.transition-all {
  transition-property: all;
  transition-duration: 0.2s;
  transition-timing-function: ease;
}
/* do not apply this on the <body> tag */
:not(body).rise-enabled {
  background-color: #fff;
}
.rise-enabled {
  transition-property: all;
  transition-duration: 0.2s;
  transition-timing-function: ease;
  /* Reveal-specific */
  /* Notebook-specific */
  /* chalkboard plugin and RISE buttons */
  /* button toolbar */
  /* use !important to override stuff that
    would normally be configured through
    chalkboard options */
}
.rise-enabled .reveal {
  /* element-overrides */
}
.rise-enabled .reveal.fade {
  transition-property: all;
  transition-duration: 0.2s;
  transition-timing-function: ease;
  opacity: 1;
}
.rise-enabled .reveal pre {
  transition-property: all;
  transition-duration: 0.2s;
  transition-timing-function: ease;
  font-size: 90%;
  width: 100%;
  box-shadow: 0px 0px 0px rgba(0, 0, 0, 0);
  overflow: hidden;
}
.rise-enabled .reveal section img {
  transition-property: all;
  transition-duration: 0.2s;
  transition-timing-function: ease;
  border: 0px solid black;
  box-shadow: 0 0 10px rgba(0, 0, 0, 0);
  transition: none;
}
.rise-enabled .reveal blockquote p:first-child,
.rise-enabled .reveal blockquote p:last-child {
  display: inherit;
}
.rise-enabled .reveal a {
  color: #727272;
}
.rise-enabled .reveal:hover {
  color: #727272;
}
.rise-enabled .reveal .slides {
  transition-property: all;
  transition-duration: 0.2s;
  transition-timing-function: ease;
  text-align: left;
  font-size: 160%;
}
.rise-enabled .reveal .slide-number {
  transition-property: all;
  transition-duration: 0.2s;
  transition-timing-function: ease;
  right: 3.3em;
  bottom: 3.9em;
  /* more harmful than helpful with reveal 3.8.0
      font-size: 100%;
    */
  color: #727272;
  background-color: transparent;
}
.rise-enabled .reveal .controls .navigate-left,
.rise-enabled .reveal .controls .navigate-left.enabled {
  border-right-color: #727272;
}
.rise-enabled .reveal .controls .navigate-left.enabled:hover,
.rise-enabled .reveal .controls .navigate-left.enabled.enabled:hover {
  border-right-color: #dfdfdf;
}
.rise-enabled .reveal .controls .navigate-right,
.rise-enabled .reveal .controls .navigate-right.enabled {
  border-left-color: #727272;
}
.rise-enabled .reveal .controls .navigate-right.enabled:hover,
.rise-enabled .reveal .controls .navigate-right.enabled.enabled:hover {
  border-left-color: #dfdfdf;
}
.rise-enabled .reveal .controls .navigate-up,
.rise-enabled .reveal .controls .navigate-up.enabled {
  border-bottom-color: #727272;
}
.rise-enabled .reveal .controls .navigate-up.enabled:hover,
.rise-enabled .reveal .controls .navigate-up.enabled.enabled:hover {
  border-bottom-color: #dfdfdf;
}
.rise-enabled .reveal .controls .navigate-down,
.rise-enabled .reveal .controls .navigate-down.enabled {
  border-top-color: #727272;
}
.rise-enabled .reveal .controls .navigate-down.enabled:hover,
.rise-enabled .reveal .controls .navigate-down.enabled.enabled:hover {
  border-top-color: #dfdfdf;
}
.rise-enabled .reveal .progress {
  margin-bottom: 0;
}
.rise-enabled .reveal .progress span {
  background: #727272;
}
.rise-enabled .reveal .reveal-skip,
.rise-enabled .reveal .reveal-notes {
  display: none;
}
.rise-enabled #notebook {
  overflow: hidden;
}
.rise-enabled #notebook.reveal {
  padding-top: 0px;
}
.rise-enabled #notebook-container {
  transition-property: all;
  transition-duration: 0.2s;
  transition-timing-function: ease;
  background-color: transparent;
  box-shadow: none;
  width: 100%;
  height: 100% !important;
}
.rise-enabled.ctb_global_show .ctb_hideshow.ctb_show {
  display: none;
}
.rise-enabled.ctb_global_show .ctb_show ~ .text_cell_render {
  border-style: none;
}
.rise-enabled .celltoolbar {
  height: inherit;
}
.rise-enabled .celltoolbar select,
.rise-enabled .celltoolbar .btn {
  height: inherit;
  font-size: inherit;
}
.rise-enabled .widget-area .form-control,
.rise-enabled .widget-area .dropdown-menu,
.rise-enabled .widget-area .btn {
  font-size: inherit;
}
.rise-enabled .widget-area .dropdown-menu {
  margin: 0px;
  /* cancel the settings from reveal */
}
.rise-enabled .cell li {
  line-height: 160%;
}
.rise-enabled .text_cell {
  font-size: 160%;
}
.rise-enabled .text_cell .prompt {
  display: none;
}
.rise-enabled .text_cell.rendered .rendered_html {
  overflow-x: hidden;
  overflow-y: hidden;
}
.rise-enabled .prompt {
  transition-property: all;
  transition-duration: 0.2s;
  transition-timing-function: ease;
  font-size: 90%;
}
.rise-enabled .output_prompt {
  transition-property: all;
  transition-duration: 0.2s;
  transition-timing-function: ease;
  margin-top: 2px;
}
.rise-enabled .output_subarea {
  transition-property: all;
  transition-duration: 0.2s;
  transition-timing-function: ease;
  margin-top: 0.05em;
}
.rise-enabled .CodeMirror {
  transition-property: all;
  transition-duration: 0.2s;
  transition-timing-function: ease;
  font-size: 100%;
  margin-bottom: 1ex;
}
.rise-enabled a.anchor-link {
  display: none;
}
.rise-enabled html {
  overflow-y: auto;
}
.rise-enabled .dialog kbd {
  padding: .1em .6em;
  border: 1px solid #ccc;
  font-size: 11px;
  background-color: #f7f7f7;
  color: #333;
  box-shadow: 0 1px 0 rgba(0, 0, 0, 0.2), 0 0 0 2px #fff inset;
  border-radius: 3px;
  display: inline-block;
  margin: 0 .1em;
  text-shadow: 0 1px 0 #fff;
  line-height: 1.4;
  white-space: nowrap;
}
.rise-enabled #header {
  transition-property: all;
  transition-duration: 0.2s;
  transition-timing-function: ease;
  top: -200px;
  opacity: 0;
}
.rise-enabled #site {
  transition-property: all;
  transition-duration: 0.2s;
  transition-timing-function: ease;
  height: 100% !important;
}
.rise-enabled #ipython-main-app {
  position: static;
}
.rise-enabled ::-webkit-scrollbar {
  width: 6px;
  height: 6px;
}
.rise-enabled ::-webkit-scrollbar * {
  background: transparent;
}
.rise-enabled ::-webkit-scrollbar-thumb {
  background: #727272 !important;
}
.rise-enabled #notescanvas {
  position: absolute;
  top: 0;
}
.rise-enabled #help_b,
.rise-enabled #exit_b,
.rise-enabled .chalkboard-button {
  color: #727272;
  position: fixed !important;
  z-index: 30;
  cursor: pointer;
}
.rise-enabled #exit_b {
  top: 0.55em;
  left: 0.5em;
}
.rise-enabled #help_b {
  left: 0.6em ;
  bottom: 0.55em;
}
.rise-enabled .chalkboard-button i {
  display: block !important;
  bottom: 1em !important;
  font-size: 1.8em !important;
  color: #727272 !important;
}
.rise-enabled #toggle-chalkboard.chalkboard-button {
  left: 4em !important;
}
.rise-enabled #toggle-notes {
  left: 7em !important;
}
.rise-enabled #notescanvas {
  display: block !important;
}
.rise-enabled #chalkboard {
  display: block !important;
}
#header {
  transition-property: all;
  transition-duration: 0.2s;
  transition-timing-function: ease;
  opacity: 1;
  top: 0px;
}
#notebook-container {
  transition-property: all;
  transition-duration: 0.2s;
  transition-timing-function: ease;
}
#notebook {
  transition-property: all;
  transition-duration: 0.2s;
  transition-timing-function: ease;
}
.chalkboard-button i {
  display: none;
}
#notescanvas {
  display: none;
}
#chalkboard {
  display: none;
}
/* fix for issue #425 */
.rise-enabled .rendered_html table {
  font-size: 75%;
}
/* fix for reveal dark themes */
body.rise-enabled.theme-black div.rendered_html tbody tr:nth-child(even),
body.rise-enabled.theme-black div.rendered_html thead th,
body.rise-enabled.theme-black div.input_prompt,
body.rise-enabled.theme-black div.output_text > pre,
body.rise-enabled.theme-black div.rendered_html,
body.rise-enabled.theme-black div.text_cell_render {
  color: white;
}
body.rise-enabled.theme-blood div.rendered_html tbody tr:nth-child(even),
body.rise-enabled.theme-blood div.rendered_html thead th,
body.rise-enabled.theme-blood div.input_prompt,
body.rise-enabled.theme-blood div.output_text > pre,
body.rise-enabled.theme-blood div.rendered_html,
body.rise-enabled.theme-blood div.text_cell_render,
body.rise-enabled.theme-league div.rendered_html tbody tr:nth-child(even),
body.rise-enabled.theme-league div.rendered_html thead th,
body.rise-enabled.theme-league div.input_prompt,
body.rise-enabled.theme-league div.output_text > pre,
body.rise-enabled.theme-league div.rendered_html,
body.rise-enabled.theme-league div.text_cell_render,
body.rise-enabled.theme-night div.rendered_html tbody tr:nth-child(even),
body.rise-enabled.theme-night div.rendered_html thead th,
body.rise-enabled.theme-night div.input_prompt,
body.rise-enabled.theme-night div.output_text > pre,
body.rise-enabled.theme-night div.rendered_html,
body.rise-enabled.theme-night div.text_cell_render {
  color: #eeeeee;
}
body.rise-enabled.theme-moon div.rendered_html tbody tr:nth-child(even),
body.rise-enabled.theme-moon div.rendered_html thead th,
body.rise-enabled.theme-moon div.input_prompt,
body.rise-enabled.theme-moon div.output_text > pre,
body.rise-enabled.theme-moon div.rendered_html,
body.rise-enabled.theme-moon div.text_cell_render {
  color: #eee8d5;
}
